/* Andrey N. Kuznetsov, Almaty, 2012 *
 * For educational purposes.         *
 * Under GPL 2.0                     */
//---------------------------------------------------------------------
#include "TCentralComponent.h"
//---------------------------------------------------------------------
#define tr(s) String::fromUTF8(s)
//---------------------------------------------------------------------
TCentralComponent::TCentralComponent() : Component("CentralComponent"),
                                         pMIDIDevicesBox(0),
                                         pPlayButton(0)
{
    pMIDIDevicesBox = new ComboBox(L"MIDIDevicesBox");
    pMIDIDevicesBox->setEditableText(false);
    pMIDIDevicesBox->setJustificationType(Justification::centredLeft);
    pMIDIDevicesBox->setTextWhenNothingSelected(tr("MIDI устройство не выбрано"));
    pMIDIDevicesBox->setTextWhenNoChoicesAvailable(tr("Нет доступных устройств"));
    pMIDIDevicesBox->addListener(this);
    addAndMakeVisible(pMIDIDevicesBox);

    pInstrumentsBox = new ComboBox(L"InstrumentsBox");
    pInstrumentsBox->setEditableText(false);
    pInstrumentsBox->setJustificationType(Justification::centredLeft);
    pInstrumentsBox->setTextWhenNothingSelected(tr("Нет выбранных инструментов"));
    pInstrumentsBox->addListener(this);
    pInstrumentsBox->setEnabled(false);
    addAndMakeVisible(pInstrumentsBox);

    // Добавляем инструменты в выпадающий список
    AddInstruments();

    pPlayButton = new TextButton(L"PlayButton");
    pPlayButton->setButtonText(tr("Играть"));
    pPlayButton->setColour(TextButton::buttonColourId, Colours::lightgreen);
    pPlayButton->setEnabled(false);
    pPlayButton->addListener(this);
    addAndMakeVisible(pPlayButton);

    // Получаем названия имеющихся устройств MIDI...
    StringArray sMIDIDevices = MidiOutput::getDevices();
    for(int i = 0; i < sMIDIDevices.size(); i++)
    {
        // и добавляем их в список
        pMIDIDevicesBox->addItem(tr("MIDI устройство: ") + sMIDIDevices[i], i + 1);
    }
    pMIDIDevicesBox->setSelectedId(1, false);

    setSize(600, 200);

    // Нота не воспроизводится
    bIsPlaying = false;
    // Инструмент по умолчанию - Acoustic Grand Piano
    iInstrument = 0;
}
//---------------------------------------------------------------------
TCentralComponent::~TCentralComponent()
{
    deleteAllChildren();

    if(pMidiOutput) delete pMidiOutput;
}
//---------------------------------------------------------------------
void TCentralComponent::paint(Graphics& Canvas)
{
    Canvas.fillAll(Colours::azure);
}
//---------------------------------------------------------------------
void TCentralComponent::resized()
{
    pMIDIDevicesBox->setBounds(10, 10, proportionOfWidth(0.9500f), 25);
    pInstrumentsBox->setBounds(10, 45, proportionOfWidth(0.9500f), 25);
    pPlayButton->setBounds(10, 140, 150, 25);
}
//---------------------------------------------------------------------
void TCentralComponent::comboBoxChanged(ComboBox* pComboBox)
{
    if(pComboBox == pMIDIDevicesBox)
    {
        // Открываем выбранное пользователем устройство MIDI
        pMidiOutput = MidiOutput::openDevice(pMIDIDevicesBox->getSelectedId() - 1);
        // Если нет доступных устройств, выводим сообщение об этом
        if(pMidiOutput == NULL)
        {
            AlertWindow::showMessageBox(AlertWindow::WarningIcon, tr("Ошибка"), tr("Невозможно открыть MIDI устройство!"), tr("Принять"), 0);
            pPlayButton->setEnabled(false);
            pInstrumentsBox->setEnabled(false);
        }
        else 
        {
            pPlayButton->setEnabled(true);
            // Иначе даём пользователю выбрать инструмент
            pInstrumentsBox->setEnabled(true);
        }
    }
    else if(pComboBox == pInstrumentsBox)
    {
        // Пользователь выбрал инструмент
        iInstrument = pInstrumentsBox->getSelectedId();
    }
}
//---------------------------------------------------------------------
void TCentralComponent::buttonClicked(Button* pButton)
{
    if(pButton == pPlayButton)
    {
        if(bIsPlaying == false)
        {
            Play();
            pPlayButton->setButtonText(tr("Остановить"));
            pPlayButton->setColour(TextButton::buttonColourId, Colours::pink);
            bIsPlaying = true;
        }
        else
        {
            MidiMessage StopMessage(0x80, 60, 0);
            StopMessage.setChannel(1);
            pMidiOutput->sendMessageNow(StopMessage);
            pPlayButton->setButtonText(tr("Играть"));
            pPlayButton->setColour(TextButton::buttonColourId, Colours::lightgreen);
            bIsPlaying = false;
        }
    }
}
//---------------------------------------------------------------------
inline void TCentralComponent::AddInstruments()
{
    pInstrumentsBox->addSectionHeading("Piano Chrom Percussion");
    pInstrumentsBox->addItem(L"Bright Acoustic Piano 9 Glockenspiel", 1);
    pInstrumentsBox->addItem(L"Electric Grand Piano 10 Music Box", 2);
    pInstrumentsBox->addItem(L"Honky-tonk Piano 11 Vibraphone", 3);
    pInstrumentsBox->addItem(L"Electric Piano 1 12 Marimba", 4);
    pInstrumentsBox->addItem(L"Electric Piano 2 13 Xylophone", 5);
    pInstrumentsBox->addItem(L"Harpsichord 14 Tubular Bells", 6);
    pInstrumentsBox->addItem(L"Clavinet 15 Dulcimer", 7);
    pInstrumentsBox->addSectionHeading("Organ Guitar");
    pInstrumentsBox->addItem(L"Drawbar Organ 24 Acoustic Guitar (nylon)", 16);
    pInstrumentsBox->addItem(L"Percussive Organ 25 Acoustic Guitar (steel)", 17);
    pInstrumentsBox->addItem(L"Rock Organ 26 Electric Guitar (jazz)", 18);
    pInstrumentsBox->addItem(L"Church Organ 27 Electric Guitar (clean)", 19);
    pInstrumentsBox->addItem(L"Reed Organ 28 Electric Guitar (muted)", 20);
    pInstrumentsBox->addItem(L"Accordion 29 Overdriven Guitar", 21);
    pInstrumentsBox->addItem(L"Harmonica 30 Distortion Guitar", 22);
    pInstrumentsBox->addItem(L"Tango Accordion 31 Guitar Harmonics", 23);
    pInstrumentsBox->addSectionHeading("Bass Strings");
    pInstrumentsBox->addItem(L"Acoustic Bass 40 Violin", 32);
    pInstrumentsBox->addItem(L"Electric Bass (finger) 41 Viola", 33);
    pInstrumentsBox->addItem(L"Electric Bass (pick) 42 Cello", 34);
    pInstrumentsBox->addItem(L"Fretless Bass 43 Contrabass", 35);
    pInstrumentsBox->addItem(L"Slap Bass 1 44 Tremolo Strings", 36);
    pInstrumentsBox->addItem(L"Slap Bass 2 45 Pizzicato Strings", 37);
    pInstrumentsBox->addItem(L"Synth Bass 1 46 Orchestral Harp", 38);
    pInstrumentsBox->addItem(L"Synth Bass 2 47 Timpani", 39);
    pInstrumentsBox->addSectionHeading("Ensemble Brass");
    pInstrumentsBox->addItem(L"String Ensemble 1 56 Trumpet", 48);
    pInstrumentsBox->addItem(L"String Ensemble 2 57 Trombone", 49);
    pInstrumentsBox->addItem(L"Synth Strings 1 58 Tuba", 50);
    pInstrumentsBox->addItem(L"Synth Strings 2 59 Muted Trumpet", 51);
    pInstrumentsBox->addItem(L"Choir Aahs 60 French Horn", 52);
    pInstrumentsBox->addItem(L"Voice Oohs 61 Brass Section", 53);
    pInstrumentsBox->addItem(L"Synth Voice 62 Synth Brass 1", 54);
    pInstrumentsBox->addItem(L"Orchestra Hit 63 Synth Brass 2", 55);
    pInstrumentsBox->addSectionHeading("Reed Pipe");
    pInstrumentsBox->addItem(L"Soprano Sax 72 Piccolo", 64);
    pInstrumentsBox->addItem(L"Alto Sax 73 Flute", 65);
    pInstrumentsBox->addItem(L"Tenor Sax 74 Recorder", 66);
    pInstrumentsBox->addItem(L"Baritone Sax 75 Pan Flute", 67);
    pInstrumentsBox->addItem(L"Oboe 76 Bottle Blow", 68);
    pInstrumentsBox->addItem(L"English Horn 77 Shakuhachi", 69);
    pInstrumentsBox->addItem(L"Bassoon 78 Whistle", 70);
    pInstrumentsBox->addItem(L"Clarinet 79 Ocarina", 71);
    pInstrumentsBox->addSectionHeading("Synth Lead Synth Pad");
    pInstrumentsBox->addItem(L"Lead 1 (square) 88 Pad 1 (new age)", 80);
    pInstrumentsBox->addItem(L"Lead 2 (sawtooth) 89 Pad 2 (warm)", 81);
    pInstrumentsBox->addItem(L"Lead 3 (calliope) 90 Pad 3 (polysynth)", 82);
    pInstrumentsBox->addItem(L"Lead 4 (chiff) 91 Pad 4 (choir)", 83);
    pInstrumentsBox->addItem(L"Lead 5 (charang) 92 Pad 5 (bowed)", 84);
    pInstrumentsBox->addItem(L"Lead 6 (voice) 93 Pad 6 (metallic)", 85);
    pInstrumentsBox->addItem(L"Lead 7 (fifths) 94 Pad 7 (halo)", 86);
    pInstrumentsBox->addItem(L"Lead 8 (bass + lead) 95 Pad 8 (sweep)", 87);
    pInstrumentsBox->addSectionHeading("SSynth Effects Ethnic");
    pInstrumentsBox->addItem(L"FX 1 (rain) 104 Sitar", 96);
    pInstrumentsBox->addItem(L"FX 2 (soundtrack) 105 Banjo", 97);
    pInstrumentsBox->addItem(L"FX 3 (crystal) 106 Shamisen", 98);
    pInstrumentsBox->addItem(L"FX 4 (atmosphere) 107 Koto", 99);
    pInstrumentsBox->addItem(L"FX 5 (brightness) 108 Kalimba", 100);
    pInstrumentsBox->addItem(L"FX 6 (goblins) 109 Bagpipe", 101);
    pInstrumentsBox->addItem(L"FX 7 (echoes) 110 Fiddle", 102);
    pInstrumentsBox->addItem(L"FX 8 (sci-fi) 111 Shanai", 103);
    pInstrumentsBox->addSectionHeading("Percussive Sound Effects");
    pInstrumentsBox->addItem(L"Tinkle Bell 120 Guitar Fret Noise", 112);
    pInstrumentsBox->addItem(L"Agogo 121 Breath Noise", 113);
    pInstrumentsBox->addItem(L"Steel Drums 122 Seashore", 114);
    pInstrumentsBox->addItem(L"Woodblock 123 Bird Tweet", 115);
    pInstrumentsBox->addItem(L"Taiko Drum 124 Telephone Ring", 116);
    pInstrumentsBox->addItem(L"Melodic Tom 125 Helicopter", 117);
    pInstrumentsBox->addItem(L"Synth Drum 126 Applause", 118);
    pInstrumentsBox->addItem(L"Reverse Cymbal 127 Gunshot", 119);
}
//---------------------------------------------------------------------
void TCentralComponent::Play()
{
    pMidiOutput->sendMessageNow(MidiMessage::programChange(1, iInstrument));
    // 100 - скорость
    MidiMessage PlayMessage(0x90, 60, 100, 0);
    // Выбираем канал
    PlayMessage.setChannel(1);
    // Отправляем сообщение
    pMidiOutput->sendMessageNow(PlayMessage);
}
//---------------------------------------------------------------------
