#include "../JuceLibraryCode/JuceHeader.h"
#include "MidiPlayerEngine.h"
//---------------------------------------------------------------------
#define tr(s) String::fromUTF8(s)
//---------------------------------------------------------------------
class TinyDisplay : public Component,
                    public ButtonListener,
                    public SliderListener,
                    public ChangeListener
{
public:
    TinyDisplay() : mPlayBtn(tr("Играть")),
                    mStopBtn(tr("Остановить")),
                    mOpenBtn(tr("Открыть...")),
                    mSlider (L"Seek")
    {
        mPlayBtn.setBounds(10, 10, 150, 30);
        mPlayBtn.addListener(this);
        mStopBtn.setBounds(170, 10, 150, 30);
        mStopBtn.addListener(this);
        mOpenBtn.setBounds(340, 10, 150, 30);
        mOpenBtn.addListener(this);
        
        mSlider.setBounds(10, 50, 480, 30);
        mSlider.setTextBoxStyle(Slider::NoTextBox, true, 0, 0);
        mSlider.addListener(this);
        
        addAndMakeVisible(&mPlayBtn);
        addAndMakeVisible(&mStopBtn);
        addAndMakeVisible(&mOpenBtn);
        addAndMakeVisible(&mSlider);
        
        mSlider.setEnabled(false);
        mStopBtn.setEnabled(false);
        mPlayBtn.setEnabled(false);
        
        // Это платформ-зависимый (Linux) метод. Он не работает под Windows.
        //midiOut = MidiOutput::createNewDevice(JUCEApplication::getInstance()->getApplicationName());
        
        // Получаем названия имеющихся устройств MIDI...
        StringArray sMIDIDevices = MidiOutput::getDevices();
        if(sMIDIDevices.size() == 1) midiOut = MidiOutput::openDevice(0);
        else midiOut = MidiOutput::openDevice(1);
        engine  = 0;
    }
    
    void sliderValueChanged(Slider* slider) {
        if (engine)
            engine->seek(slider->getValue());
    }
    
    void buttonClicked(Button* button) {
        if (button == (Button*)&mOpenBtn) {
            FileChooser fc(tr("Открыть файл MIDI"),
                           File::getSpecialLocation(File::userMusicDirectory),
                           "*.mid;*.kar");
            if (!fc.browseForFileToOpen())
                return;
            
            if (engine) {
                engine->stop();
                delete engine;
            }
            
            engine = new MidiPlayerEngine(File(fc.getResult()).createInputStream(), midiOut);
            engine->addChangeListener(this);
            mPlayBtn.setEnabled(true);
            mPlayBtn.setToggleState(false, false);
            mStopBtn.setEnabled(true);
            mSlider.setEnabled(true);
            mSlider.setRange(0, engine->getLength());
            Logger::writeToLog(String(L"Total length = ") + String(engine->getLength()));
            
        } else if (button == (Button*)&mStopBtn) {
            engine->stop();
            mPlayBtn.setToggleState(false, false);
            mSlider.setValue(0, false);
            mSlider.setEnabled(false);
            
        } else if (button == (Button*)&mPlayBtn) {
            if (mPlayBtn.getToggleState())
                engine->play();
            else
                engine->pause();
                engine->reset();
        }
    }
    
    void changeListenerCallback(ChangeBroadcaster* obj) {
        MidiPlayerEngine* k = (MidiPlayerEngine*)obj;
        mSlider.setValue(k->getSeekPosition());
        Logger::writeToLog(String(L"Position ") + String(k->getSeekPosition()));
        repaint();
    }

protected:
    ToggleButton    mPlayBtn;
    TextButton      mStopBtn;
    TextButton      mOpenBtn;
    Slider          mSlider;
    MidiPlayerEngine* engine;
    MidiOutput*     midiOut;
};
